#include "LZActorValuePercentageFix.h"

namespace lznsAVFix {
	static const UInt32 Condition_GetAVPercent_Base = 0x5067DF;
	static const UInt32 Condition_GetAVPercent_Return = Condition_GetAVPercent_Base + 0x5;

	static const UInt32 Condition_GetStaminaPercentage_Base = 0x506A0C;
	static const UInt32 Condition_GetStaminaPercentage_Return = Condition_GetStaminaPercentage_Base + 0x5;

	static const UInt32 Condition_GetHealthPercentage_Base = 0x506A8C;
	static const UInt32 Condition_GetHealthPercentage_Return = Condition_GetHealthPercentage_Base + 0x5;

	static const UInt32 Condition_GetAVPercentagePapyrus_Base = 0x8DA706;
	static const UInt32 Condition_GetAVPercentagePapyrus_Return = Condition_GetAVPercentagePapyrus_Base + 0x5;

	/**
	 * Obtains the actor value percent by incorporating the temporary modifier (normally omitted by the game).
	 * The offsets defined above account for all Condition Functions in the Creation Kit, and the associated item for Papyrus.
	 */
	float GetAVPercent(ActorExtended* actor, UInt32 avIndex) {
		if (actor) {
			const float max = actor->actorValueOwner.GetMaximum(avIndex);
			const float temp = CALL_MEMBER_FN(actor, GetActorValueModifier)(1, avIndex);
			const float sum = max + temp;
			if (sum > 0) {
				return actor->actorValueOwner.GetCurrent(avIndex) / sum;
			}
		}
		return 1.0f;
	}

	/**
	 * Obtains the actor value's maximum value by incorporating the temporary modifier (normally omitted by the game).
	 * May return a negative value or zero.
	 * Always returns -1 if actor is not defined.
	 */
	float GetAVMaximum(ActorExtended* actor, UInt32 avIndex) {
		if (actor) {
			const float max = actor->actorValueOwner.GetMaximum(avIndex);
			const float temp = CALL_MEMBER_FN(actor, GetActorValueModifier)(1, avIndex);
			return max + temp;
		}
		return -1.0f;
	}

	/** Overrides the engine behavior for GetActorValuePercent. */
	static void __declspec(naked) hookConditionAVPercent() {
		__asm {
			push eax
			push ecx
			call GetAVPercent
			add esp, 0x10

			jmp Condition_GetAVPercent_Return
		}
	}

	/** Overrides the engine behavior for GetStaminaPercentage. */
	static void __declspec(naked) hookConditionStaminaPercentage() {
		__asm {
			push 0x1A
			push ecx
			call GetAVPercent
			add esp, 0xC

			jmp Condition_GetStaminaPercentage_Return
		}
	}

	/** Overrides the engine behavior for GetStaminaPercentage. */
	static void __declspec(naked) hookConditionHealthPercentage() {
		__asm {
			push 0x18
			push ecx
			call GetAVPercent
			add esp, 0xC

			jmp Condition_GetHealthPercentage_Return
		}
	}

	/** Overrides the engine behavior for the GetActorValuePercentage function in Papyrus. */
	static void __declspec(naked) hookConditionAVPercentPapyrus() {
		__asm {
			push eax
			push ecx
			call GetAVPercent
			add esp, 0x10

			jmp Condition_GetAVPercentagePapyrus_Return
		}
	}

	/** Obtains the actor's temporary AV modifier. */
	float GetAVModifier(StaticFunctionTag* base, ActorExtended* actor, UInt32 modType, BSFixedString actorValue) {
		if (actor) {
			UInt32 av = ActorValueList::ResolveActorValueByName(actorValue.data);
			if (av != 255) {
				return CALL_MEMBER_FN(actor, GetActorValueModifier)(modType, av);
			}
		}
		return 0.0f;
	}

	/** Installs the core mod. */
	void Install() {
		WriteRelCall(Condition_GetAVPercent_Base, (UInt32) hookConditionAVPercent);
		WriteRelCall(Condition_GetStaminaPercentage_Base, (UInt32) hookConditionStaminaPercentage);
		WriteRelCall(Condition_GetHealthPercentage_Base, (UInt32) hookConditionHealthPercentage);
		WriteRelCall(Condition_GetAVPercentagePapyrus_Base, (UInt32) hookConditionAVPercentPapyrus);
	}

	/** Registers all new Papyrus functions into the plugin. */
	bool RegisterFuncs(VMClassRegistry* registry) {
		registry->RegisterFunction(
			new NativeFunction3 <StaticFunctionTag, float, ActorExtended*, UInt32, BSFixedString>("GetAVModifier", "LZAVPF", lznsAVFix::GetAVModifier, registry));

		return true;
	}
}
