#include <shlwapi.h>
#pragma comment( lib, "shlwapi.lib" )

#include "iniBase.h"
#include "mini.h"

/**
 * Loads the ini file from disk, return false if loading failed or the file does not exist.
 * Can only be called once.
 */
bool IniFile::load() {
	if (!filePath.empty()) {
		return false;
	}
	filePath = generatePath();
	setDefaults();

	mINI::INIFile file(filePath);
	mINI::INIStructure ini;
	if (!file.read(ini)) {
		return false;
	}

	for (auto const& it : ini) {
		auto section = it.first;
		toLower(section);

		auto const& collection = it.second;
		for (auto const& it2 : collection) {
			auto key = it2.first;
			auto value = it2.second;

			toLower(key);
			putSetting(section, key, value);
		}
	}
	finishLoading();
	return true;
}

/** Creates the path for the initialization file. */
std::string IniFile::generatePath() {
	std::string result;

	char buffer[MAX_PATH];
	GetModuleFileName(nullptr, buffer, MAX_PATH);
	PathRemoveFileSpec(buffer);
	result = buffer;

	result += "\\Data\\SKSE\\Plugins\\";
	result += getFilename();
	return result;
}

/** Converts the requested value into an integer. Invalid arguments produce 0. */
UInt32 IniFile::toInt(std::string value) {
	UInt32 result;
	try {
		result = std::stoi(value);
	} catch (...) {
		result = 0;
	}
	return result;
}

/** Converts the requested value into a float. Invalid arguments produce 0.0. */
float IniFile::toFloat(std::string value) {
	float result;
	try {
		result = std::stof(value);
	} catch (...) {
		result = 0;
	}
	return result;
}

/**
 * Converts the requested value into a boolean.
 * The string "true" and any nonzero integer value are each considered true; everything else is false.
 * The value will be converted to lower-case.
 */
bool IniFile::toBool(std::string value) {
	toLower(value);
	if (value == "true") {
		return true;
	}

	try {
		int result = std::stoi(value);
		return result != 0;
	} catch (...) {
		return false;
	}
}

/** Converts the provided string to all lower-case. */
void IniFile::toLower(std::string& str) {
	for (auto& c : str) {
		c = std::tolower(c);
	}
}
