#include "skse/PluginAPI.h"		// super
#include "skse/skse_version.h"	// What version of SKSE is running?
#include <shlobj.h>				// CSIDL_MYCODUMENTS

#include "LZHealthGating.h"
#include "LZIOUtil.h"
#include "LZStringUtil.h"
#include "LZJumpBonusFix.h"

static PluginHandle					g_pluginHandle = kPluginHandle_Invalid;
static SKSEPapyrusInterface         * g_papyrus = NULL;

extern "C"	{

	bool SKSEPlugin_Query(const SKSEInterface * skse, PluginInfo * info)	{	// Called by SKSE to learn about this plugin and check that it's safe to load it
		gLog.OpenRelative(CSIDL_MYDOCUMENTS, "\\My Games\\Skyrim\\SKSE\\LZEnhancements.log");
		gLog.SetPrintLevel(IDebugLog::kLevel_Error);
		gLog.SetLogLevel(IDebugLog::kLevel_DebugMessage);

		_MESSAGE("LZEnhancements");

		// populate info structure
		info->infoVersion =	PluginInfo::kInfoVersion;
		info->name =		"LZEnhancements";
		info->version =		1;

		// store plugin handle so we can identify ourselves later
		g_pluginHandle = skse->GetPluginHandle();

		if(skse->isEditor)
		{
			_MESSAGE("loaded in editor, marking as incompatible");

			return false;
		}
		else if(skse->runtimeVersion != RUNTIME_VERSION_1_9_32_0)
		{
			_MESSAGE("unsupported runtime version %08X", skse->runtimeVersion);

			return false;
		}

		// ### do not do anything else in this callback
		// ### only fill out PluginInfo and return true/false

		// supported runtime version
		return true;
	}

	bool SKSEPlugin_Load(const SKSEInterface * skse)	{	// Called by SKSE to load this plugin
		lznsAVFix::Install();
		_MESSAGE("LZ Actor Value Percentage Fix loaded");

		lznsJumpBonusFix::Install();
		_MESSAGE("LZ Jump Bonus Fix loaded");

		ProjectIni* iniFile = new ProjectIni();
		bool foundSettings = iniFile->load();
		const bool canUseHealthGating = (foundSettings && iniFile->isEnabled);

		if (canUseHealthGating) {
			lznsHealthGating::Install(iniFile);
			_MESSAGE("LZ Health Gating loaded; current status ENABLED");
		} else {
			_MESSAGE("LZ Health Gating loaded; current status DISABLED");
		}


		g_papyrus = (SKSEPapyrusInterface*)skse->QueryInterface(kInterface_Papyrus);

		bool registering = g_papyrus->Register(lznsAVFix::RegisterFuncs);
		_MESSAGE("LZ Actor Value Percentage Fix functions registered");

		registering &= g_papyrus->Register(lznsIOUtil::RegisterFuncs);
		_MESSAGE("LZ IO Utils functions registered");

		registering &= g_papyrus->Register(lznsStringUtil::RegisterFuncs);
		_MESSAGE("LZ String Utils functions registered");

		if (canUseHealthGating) {
			registering &= g_papyrus->Register(lznsHealthGating::RegisterFuncs);
			_MESSAGE("LZ Health Gating functions registered");
		}

		if (registering) {
			_MESSAGE("Register Succeeded");
		}
		return true;
	}
};