#include "projectIni.h"

std::string ProjectIni::getFilename() {
	return "lzEnhancements.ini";
}

void ProjectIni::setDefaults() {
	isEnabled = false;
	enablePhysical = true;
	enableMagic = true;
	forMainPlayerOnly = true;
	requireBelowAttackerHealth = false;
	requireBelowAttackerStamina = false;
	minHealthPercent = 0.9f;
	resultHealthPercent = 0.1f;
	attackerHealthPercent = 0.0f;
	attackerStaminaPercent = 0.0f;
	immunityTimeMillis = 500;
	cleanupPollingTimeMillis = 180000;
}

void ProjectIni::putSetting(std::string section, std::string key, std::string value) {
	if (key == "isenabled") {
		isEnabled = toBool(value);
		_MESSAGE(("enabled:" + std::to_string(isEnabled)).c_str());

	} else if (key == "enablephysical") {
		enablePhysical = toBool(value);
		_MESSAGE(("enablephysical:" + std::to_string(enablePhysical)).c_str());

	} else if (key == "enablemagic") {
		enableMagic = toBool(value);
		_MESSAGE(("enablemagic:" + std::to_string(enableMagic)).c_str());

	} else if (key == "formainplayeronly") {
		forMainPlayerOnly = toBool(value);
		_MESSAGE(("forMainPlayerOnly:" + std::to_string(forMainPlayerOnly)).c_str());

	} else if (key == "requirebelowattackerhealth") {
		requireBelowAttackerHealth = toBool(value);
		_MESSAGE(("requireBelowAttackerHealth:" + std::to_string(requireBelowAttackerHealth)).c_str());

	} else if (key == "requirebelowattackerstamina") {
		requireBelowAttackerStamina = toBool(value);
		_MESSAGE(("requireBelowAttackerStamina:" + std::to_string(requireBelowAttackerStamina)).c_str());

	} else if (key == "swapattackerstaminamagicka") {
		swapAttackerStaminaMagicka = toBool(value);
		_MESSAGE(("swapAttackerStaminaMagicka:" + std::to_string(swapAttackerStaminaMagicka)).c_str());

	} else if (key == "minhealthpercent") {
		minHealthPercent = toFloat(value);
		_MESSAGE(("minHealthPercent:" + std::to_string(minHealthPercent)).c_str());

	} else if (key == "resulthealthpercent") {
		resultHealthPercent = toFloat(value);
		_MESSAGE(("resultHealthPercent:" + std::to_string(resultHealthPercent)).c_str());

	} else if (key == "attackerhealthpercent") {
		attackerHealthPercent = toFloat(value);
		_MESSAGE(("attackerHealthPercent:" + std::to_string(attackerHealthPercent)).c_str());

	} else if (key == "attackerstaminapercent") {
		attackerStaminaPercent = toFloat(value);
		_MESSAGE(("attackerStaminaPercent:" + std::to_string(attackerStaminaPercent)).c_str());

	} else if (key == "immunitytimemillis") {
		immunityTimeMillis = toInt(value);

	} else if (key == "cleanuppollingTimemillis") {
		cleanupPollingTimeMillis = toInt(value);
	}
}

void ProjectIni::finishLoading() {
	if (!enablePhysical && !enableMagic) {
		isEnabled = false;
	}
	if (minHealthPercent > 1.0f || minHealthPercent <= resultHealthPercent) {
		isEnabled = false;
	}
	if (resultHealthPercent <= 0.0f || resultHealthPercent >= minHealthPercent) {
		isEnabled = false;
	}
	if (attackerHealthPercent <= 0.0f || attackerHealthPercent > 1.0f) {
		requireBelowAttackerHealth = false;
	}	
	if (attackerStaminaPercent <= 0.0f || attackerStaminaPercent > 1.0f) {
		requireBelowAttackerStamina = false;
	}
	if (immunityTimeMillis < 0.0f) {
		immunityTimeMillis = 500;
	}
	if (cleanupPollingTimeMillis < immunityTimeMillis + 1000) {
		cleanupPollingTimeMillis = immunityTimeMillis + 1000;
	}
	if (forMainPlayerOnly) {
		cleanupPollingTimeMillis = -1.0f;
	}
}
