#include <mutex>
#include <condition_variable>

/** A simple semaphore implementation. */
class Semaphore {
public:
	/** Attempts to acquire this semaphore, blocking the current thread if it is not available. */
	void acquire() {
		std::unique_lock<decltype(_mutex)> lock(_mutex);
		while (_count < 0) {
			_condition.wait(lock);
		}
		_count--;
	}

	/** Releases the current semaphore lock. CANNOT be called repeatedly to generate extra permits. */
	void release() {
		std::lock_guard<decltype(_mutex)> lock(_mutex);
		_count = (std::min<unsigned long>)(_count + 1, 0);
		_condition.notify_one();
	}

	/** Acquires, then immediately releases the semaphore. */
	void waitFor() {
		acquire();
		release();
	}

private:
	std::mutex _mutex;
	std::condition_variable _condition;
	unsigned long _count = 0;
};
